@@ -71,7 +71,7 @@
 #define AST_METADATA_PROP_FLAGS(object) OBJ_PROP_NUM((object), 2)
 #define AST_METADATA_PROP_FLAGS_COMBINABLE(object) OBJ_PROP_NUM((object), 3)
 
-#define AST_CURRENT_VERSION 110
+#define AST_CURRENT_VERSION 120
 
 /* Additional flags for BINARY_OP */
 #define AST_BINARY_IS_GREATER 256
@@ -79,6 +79,7 @@
 #define AST_BINARY_BOOL_OR 258
 #define AST_BINARY_BOOL_AND 259
 #define AST_BINARY_COALESCE 260
+#define AST_BINARY_PIPE 261
 
 /* Flags for UNARY_OP to use instead of AST_SILENCE, AST_UNARY_PLUS, AST_UNARY_MINUS */
 #define AST_SILENCE 260
@@ -117,6 +118,16 @@
 # define ZEND_ENCAPS_VAR_DOLLAR_CURLY_VAR_VAR (1<<1)
 #endif
 
+#if PHP_VERSION_ID < 80400
+# define MODIFIER_PUBLIC_SET    (1 << 11)
+# define MODIFIER_PROTECTED_SET (1 << 12)
+# define MODIFIER_PRIVATE_SET   (1 << 13)
+#else
+# define MODIFIER_PUBLIC_SET    ZEND_ACC_PUBLIC_SET
+# define MODIFIER_PROTECTED_SET ZEND_ACC_PROTECTED_SET
+# define MODIFIER_PRIVATE_SET   ZEND_ACC_PRIVATE_SET
+#endif
+
 #if PHP_VERSION_ID >= 80400
 # define ZEND_DIM_ALTERNATIVE_SYNTAX (1<<1)
 #endif
@@ -226,6 +237,7 @@ static const char *binary_op_flags[] = {
 	AST_FLAG(BINARY_IS_GREATER),
 	AST_FLAG(BINARY_IS_GREATER_OR_EQUAL),
 	AST_FLAG(BINARY_SPACESHIP),
+	AST_FLAG(BINARY_PIPE),
 	NULL
 };
 
@@ -289,11 +301,22 @@ static const char *closure_use_flags[] = {
 	AST_FLAG(MODIFIER_FINAL), \
 	AST_FLAG(MODIFIER_READONLY)
 
+#define AST_ASYMMETRIC_VISIBILITY_FLAGS \
+	AST_FLAG(MODIFIER_PUBLIC_SET), \
+	AST_FLAG(MODIFIER_PROTECTED_SET), \
+	AST_FLAG(MODIFIER_PRIVATE_SET)
+
 static const char *modifier_flags[] = {
 	AST_MODIFIER_FLAGS,
 	NULL
 };
 
+static const char *property_modifier_flags[] = {
+	AST_MODIFIER_FLAGS,
+	AST_ASYMMETRIC_VISIBILITY_FLAGS,
+	NULL
+};
+
 static const char *func_flags[] = {
 	AST_MODIFIER_FLAGS,
 	AST_FLAG(FUNC_RETURNS_REF),
@@ -341,8 +364,8 @@ static const ast_flag_info flag_info[] = {
 	{ ZEND_AST_CLOSURE, 1, func_flags },
 	{ ZEND_AST_ARROW_FUNC, 1, func_flags },
 	{ ZEND_AST_PROPERTY_HOOK, 1, func_flags },
-	{ ZEND_AST_PROP_DECL, 1, modifier_flags },
-	{ ZEND_AST_PROP_GROUP, 1, modifier_flags },
+	{ ZEND_AST_PROP_DECL, 1, property_modifier_flags },
+	{ ZEND_AST_PROP_GROUP, 1, property_modifier_flags },
 	{ ZEND_AST_CLASS_CONST_DECL, 1, modifier_flags },
 	{ ZEND_AST_CLASS_CONST_GROUP, 1, modifier_flags },
 	{ ZEND_AST_TRAIT_ALIAS, 1, modifier_flags },
@@ -959,6 +982,12 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 			ast->kind = ZEND_AST_BINARY_OP;
 			ast->attr = AST_BINARY_COALESCE;
 			break;
+#if PHP_VERSION_ID >= 80500
+		case ZEND_AST_PIPE:
+			ast->kind = ZEND_AST_BINARY_OP;
+			ast->attr = AST_BINARY_PIPE;
+			break;
+#endif
 		case ZEND_AST_SILENCE:
 			ast->kind = ZEND_AST_UNARY_OP;
 			ast->attr = AST_SILENCE;
@@ -1046,6 +1075,64 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 			}
 			break;
 #endif
+#if PHP_VERSION_ID >= 80500
+		case ZEND_AST_CAST_VOID:
+			ast->kind = ZEND_AST_CAST;
+			ast->attr = IS_VOID;
+			break;
+#endif
+		case ZEND_AST_CALL:
+			if (state->version < 120) {
+				// Convert clone($expr) call to ZEND_AST_CLONE node.
+				zend_ast *name_ast = ast->child[0];
+				if (name_ast->kind != ZEND_AST_ZVAL || name_ast->attr != ZEND_NAME_FQ ||
+						Z_TYPE_P(zend_ast_get_zval(name_ast)) != IS_STRING ||
+						ast->child[1]->kind != ZEND_AST_ARG_LIST) {
+					break;
+				}
+
+				zend_ast_list *args = zend_ast_get_list(ast->child[1]);
+				if (args->children != 1) {
+					break;
+				}
+
+				zend_ast *arg = args->child[0];
+				if (arg->kind == ZEND_AST_NAMED_ARG || arg->kind == ZEND_AST_UNPACK) {
+					break;
+				}
+
+				zend_string *name = zend_ast_get_str(name_ast);
+				if (zend_string_equals_literal_ci(name, "clone")) {
+					ast_create_virtual_node(zv, ZEND_AST_CLONE, 0, arg, state);
+					return;
+				}
+				if (zend_string_equals_literal_ci(name, "exit")) {
+					ast_create_virtual_node(zv, ZEND_AST_EXIT, 0, arg, state);
+					return;
+				}
+			}
+			break;
+		case ZEND_AST_CLONE:
+		case ZEND_AST_EXIT:
+			if (state->version >= 120) {
+				// Convert to call node.
+				uint32_t lineno = zend_ast_get_lineno(ast);
+				zval name_str_zv, name_node_zv, args_zv, arg_zv;
+				if (ast->kind == ZEND_AST_CLONE) {
+					ZVAL_STR(&name_str_zv, AST_STR(str_clone));
+				} else {
+					ZVAL_STR(&name_str_zv, AST_STR(str_exit));
+				}
+				ast_to_zval(&arg_zv, ast->child[0], state);
+				ast_create_virtual_node_ex(
+						&name_node_zv, AST_NAME, ZEND_NAME_FQ, lineno, state, 1, &name_str_zv);
+				ast_create_virtual_node_ex(
+						&args_zv, ZEND_AST_ARG_LIST, 0, lineno, state, 1, &arg_zv);
+				ast_create_virtual_node_ex(
+						zv, ZEND_AST_CALL, 0, lineno, state, 2, &name_node_zv, &args_zv);
+				return;
+			}
+			break;
 	}
 
 	object_init_ex(zv, ast_node_ce);
@@ -1136,7 +1223,7 @@ static void ast_to_zval(zval *zv, zend_ast *ast, ast_state_info_t *state) {
 #endif
 }
 
-static const zend_long versions[] = {50, 60, 70, 80, 85, 90, 100, 110};
+static const zend_long versions[] = {50, 60, 70, 80, 85, 90, 100, 110, 120};
 static const size_t versions_count = sizeof(versions)/sizeof(versions[0]);
 
 static inline zend_bool ast_version_deprecated(zend_long version) {
@@ -1466,6 +1553,9 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("MODIFIER_ABSTRACT", ZEND_ACC_ABSTRACT);
 	ast_register_flag_constant("MODIFIER_FINAL", ZEND_ACC_FINAL);
 	ast_register_flag_constant("MODIFIER_READONLY", ZEND_ACC_READONLY);
+	ast_register_flag_constant("MODIFIER_PUBLIC_SET", MODIFIER_PUBLIC_SET);
+	ast_register_flag_constant("MODIFIER_PROTECTED_SET", MODIFIER_PROTECTED_SET);
+	ast_register_flag_constant("MODIFIER_PRIVATE_SET", MODIFIER_PRIVATE_SET);
 
 	ast_register_flag_constant("PARAM_MODIFIER_PUBLIC", PARAM_MODIFIER_PUBLIC);
 	ast_register_flag_constant("PARAM_MODIFIER_PROTECTED", PARAM_MODIFIER_PROTECTED);
@@ -1536,6 +1626,7 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("BINARY_IS_GREATER_OR_EQUAL", AST_BINARY_IS_GREATER_OR_EQUAL);
 	ast_register_flag_constant("BINARY_SPACESHIP", ZEND_SPACESHIP);
 	ast_register_flag_constant("BINARY_COALESCE", AST_BINARY_COALESCE);
+	ast_register_flag_constant("BINARY_PIPE", AST_BINARY_PIPE);
 
 	ast_register_flag_constant("EXEC_EVAL", ZEND_EVAL);
 	ast_register_flag_constant("EXEC_INCLUDE", ZEND_INCLUDE);
