@@ -11,6 +11,8 @@
 	X(endLineno, "endLineno") \
 	X(__declId, "__declId") \
 	X(flagsCombinable, "flagsCombinable") \
+	X(clone, "clone") \
+	X(exit, "exit") \
 	X(type, "type") \
 	X(params, "params") \
 	X(uses, "uses") \
