@@ -37,6 +37,7 @@ const AST_CLOSURE = 68;
 const AST_METHOD = 69;
 const AST_ARROW_FUNC = 71;
 const AST_CLASS = 70;
+const AST_PROPERTY_HOOK = 127;
 const AST_MAGIC_CONST = 0;
 const AST_TYPE = 1;
 const AST_CALLABLE_CONVERT = 3;
@@ -69,6 +70,7 @@ const AST_GOTO = 285;
 const AST_BREAK = 286;
 const AST_CONTINUE = 287;
 const AST_CLASS_NAME = 276;
+const AST_PROPERTY_HOOK_SHORT_BODY = 510;
 const AST_CLASS_CONST_GROUP = 777;
 const AST_DIM = 512;
 const AST_PROP = 513;
@@ -199,14 +201,14 @@ const EXEC_REQUIRE_ONCE = 16;
 const USE_NORMAL = 1;
 const USE_FUNCTION = 2;
 const USE_CONST = 4;
-const MAGIC_LINE = 379;
-const MAGIC_FILE = 380;
-const MAGIC_DIR = 381;
-const MAGIC_NAMESPACE = 386;
-const MAGIC_FUNCTION = 385;
-const MAGIC_METHOD = 384;
-const MAGIC_CLASS = 382;
-const MAGIC_TRAIT = 383;
+const MAGIC_LINE = 343;
+const MAGIC_FILE = 344;
+const MAGIC_DIR = 345;
+const MAGIC_NAMESPACE = 350;
+const MAGIC_FUNCTION = 349;
+const MAGIC_METHOD = 348;
+const MAGIC_CLASS = 346;
+const MAGIC_TRAIT = 347;
 const ARRAY_SYNTAX_LIST = 1;
 const ARRAY_SYNTAX_LONG = 2;
 const ARRAY_SYNTAX_SHORT = 3;
