@@ -7,7 +7,7 @@
 extern zend_module_entry ast_module_entry;
 #define phpext_ast_ptr &ast_module_entry
 
-#define PHP_AST_VERSION "1.1.2"
+#define PHP_AST_VERSION "1.1.3"
 
 #ifdef PHP_WIN32
 #	define PHP_AST_API __declspec(dllexport)
@@ -93,6 +93,12 @@ extern ast_str_globals str_globals;
 # define ZEND_AST_PARENT_PROPERTY_HOOK_CALL 0x2f8
 #endif
 
+/* ZEND_AST_EXIT and ZEND_AST_CLONE have been replaced with plain function call. */
+#if PHP_VERSION_ID >= 80500
+# define ZEND_AST_EXIT 0x1fd
+# define ZEND_AST_CLONE 0x1fc
+#endif
+
 /* Pretend it still exists */
 # define ZEND_AST_LIST ((1 << (ZEND_AST_IS_LIST_SHIFT + 1)) - 1)
 
